@@ -27,11 +27,11 @@ class TableRowIter():
         self._r = r
         self._c = 0
         self._tab = tab
-        
+
     def __iter__(self):
         self._c = 0
         return self
-    
+
     def __next__(self):
         if self._c >= self._tab._col:
             raise StopIteration
@@ -53,17 +53,17 @@ class TableRowOperator():
         """
         self._r = r
         self._tab = tab
-        
+
     def __getitem__(self, c):
         return self._tab.getItem(self._r, c)
-        
+
     def __setitem__(self, c, val):
         self._tab.setItem(self._r, c, val)
 
 
 class Table():
     """A static table with label text in table cells.
-    
+
     [WARNING] Don't create this class directly, use algviz.Visualizer.createTable instead.
     """
 
@@ -75,11 +75,11 @@ class Table():
             data (list(list(printable))): The initial data for table cells.
             cell_size tuple(float, float): Table cell size (width, height).
             show_index (bool): Whether to display table row and column labels.
-        
+
         Raises:
             AlgvizParamError: Table row or col number should > 0.
         """
-        if row <=0 or col <=0:
+        if row <= 0 or col <= 0:
             raise AlgvizParamError('Table row or col number should > 0.')
         if type(cell_size) != tuple or len(cell_size) < 2:
             raise AlgvizParamError('Table cell_size parameter should be <tuple(float, float)> type.')
@@ -90,12 +90,13 @@ class Table():
         self._frame_trace = list()          # Record the relevant information of the cell to be refreshed in the next frame.
         self._delay = 0                     # Animation frame delay time, used to adapt Visualizer class.
         self._next_row = 0                  # Used to mark which row the current iterated is on for the table.
-        self._cell_width = clamp(cell_size[0], kMinCellWidth, kMaxCellWidth)    # The rectangle cells width in table.
-        self._cell_height = clamp(cell_size[1], kMinCellHeight, kMaxCellHeight) # The rectangle cells width in table.
+        self._cell_width = clamp(cell_size[0], kMinCellWidth, kMaxCellWidth)        # The rectangle cells width in table.
+        self._cell_height = clamp(cell_size[1], kMinCellHeight, kMaxCellHeight)     # The rectangle cells width in table.
         self._cell_margin = 3               # The cell margin between cell and SVG side.
         self._show_index = show_index       # Wheather to show subscript of rows and columns index in table.
         self._label_font_size = 0           # The font size of the subscript labels in table.
         self._index2rect = dict()           # Map the row and column index into rectangle's gid in SVG.
+        self._items_to_update = dict()      # {key:rect_gid, val:rect_label} Cache all the items in the table to be update since last frame.
         self._data = [[None for _ in range(self._col)] for _ in range(self._row)]
         self._svg = SvgTable(self._cell_margin, self._cell_margin)
         # Copy data into table.
@@ -104,20 +105,20 @@ class Table():
                 for c in range(self._col):
                     try:
                         self._data[r][c] = data[r][c]
-                    except:
+                    except Exception:
                         self._data[r][c] = None
         # Add row and column cursor managers for table.
         self._row_cursor_mgr = _CursorManager((self._cell_width, self._cell_height),
-            self._svg, 'R', (self._cell_margin, self._cell_margin), 0)
+                                              self._svg, 'R', (self._cell_margin, self._cell_margin), 0)
         self._col_cursor_mgr = _CursorManager((self._cell_width, self._cell_height),
-            self._svg, 'D', (self._cell_margin, self._cell_margin), 0)
+                                              self._svg, 'D', (self._cell_margin, self._cell_margin), 0)
         # Initial rectangle elemenets in SVG.
         for r in range(self._row):
             for c in range(self._col):
                 self._new_rect_in_svg_(r, c)
         # Initial the subscript labels in table.
         if self._show_index:
-            self._label_font_size = int(min(12, self._cell_width/len(str(max(self._row,self._col)-1))))
+            self._label_font_size = int(min(12, self._cell_width / len(str(max(self._row, self._col) - 1))))
             self._row_index2text = dict()       # Map the row index into the row subscript's text gid in SVG.
             self._col_index2text = dict()       # Map the column index into the row subscript's text gid in SVG.
             for r in range(self._row):
@@ -125,27 +126,35 @@ class Table():
             for c in range(self._col):
                 self._new_col_index_text_in_svg_(c)
         self._update_svg_size_()
-    
 
-    def mark(self, color, r, c, hold=False):
+    def mark(self, color, r, c, hold=False, r2=None, c2=None):
         """Emphasize one cell in the table by mark it's background color.
-        
+
         Args:
             color ((R,G,B)): The background color for the marked cell. R, G, B stand for color channel for red, green, blue.
                 R,G,B should be int value and 0 <= R,G,B <= 255. eg:(0, 255, 0)
             r, c (int/Cursor): Index the cell's raw, column in the table to be marked.
             hold (bool): Whether to keep the mark color in future animation frames.
-        
+            r2, c2 (int/Cursor): If r2, c2 are not None, this will mark cells in the rectange range(r, c, r2, c2).
+
         Raises:
             RuntimeError: Index:xx type is not int or Cursor.
             RuntimeError: Table index=xxx out of range.
         """
         r = self._check_index_type_and_range_(r, self._row)
         c = self._check_index_type_and_range_(c, self._col)
-        gid = self._index2rect[(r, c)]
-        self._cell_tcs[gid].add(color)
-        self._frame_trace.append((gid, color, hold))
-    
+        if r2 is None or c2 is None:
+            gid = self._index2rect[(r, c)]
+            self._cell_tcs[gid].add(color)
+            self._frame_trace.append((gid, color, hold))
+        else:
+            r2 = self._check_index_type_and_range_(r2, self._row)
+            c2 = self._check_index_type_and_range_(c2, self._col)
+            for i in range(r, r2 + 1):
+                for j in range(c, c2 + 1):
+                    gid = self._index2rect[(i, j)]
+                    self._cell_tcs[gid].add(color)
+                    self._frame_trace.append((gid, color, hold))
 
     def removeMark(self, color):
         """Remove the mark color for cell(s).
@@ -157,14 +166,22 @@ class Table():
         for gid in self._index2rect.values():
             if self._cell_tcs[gid].remove(color):
                 self._svg.update_rect_element(gid, fill=self._cell_tcs[gid].color())
-    
+
+    def removeMarks(self, color_list):
+        """Remove the mark colors for cell(s).
+
+        Args:
+            color_list ([(R,G,B), ...]): all the colors to be removed.
+        """
+        for color in color_list:
+            self.removeMark(color)
 
     def getItem(self, r, c):
         """Get the cell value in the table.
-        
+
         Args:
             r, c (int/Cursor): Index the cell's raw, column in the table.
-        
+
         Returns:
             printable: The value in the specific cell.
 
@@ -179,11 +196,10 @@ class Table():
         r = self._check_index_type_and_range_(r, self._row)
         c = self._check_index_type_and_range_(c, self._col)
         return self._data[r][c]
-    
 
     def setItem(self, r, c, val):
         """ Get the cell value in the table.
-        
+
         Args:
             r, c (int/Cursor): Index the cell's raw, column in the table to be modified.
             val (printable): New value for the cell.
@@ -202,9 +218,8 @@ class Table():
         label = val
         if val is None:
             label = ''
-        self._svg.update_rect_element(gid, text=label)
+        self._items_to_update[gid] = label
         self._data[r][c] = val
-    
 
     def shape(self):
         """
@@ -213,7 +228,6 @@ class Table():
         """
         return (self._row, self._col)
 
-    
     def reshape(self, row, col):
         """Reshape the row and column size of this table.
         row, col (int): The new row and column number of the table.
@@ -221,7 +235,7 @@ class Table():
         Raises:
             AlgvizParamError: Table row or col number should > 0.
         """
-        if row <=0 or col <=0:
+        if row < 0 or col < 0:
             raise AlgvizParamError('Table row or col number should > 0.')
         if row > self._row:
             # Add new rows into table.
@@ -261,13 +275,12 @@ class Table():
         self._update_svg_size_()
         self._update_subscripts_position_()
 
-
     def _add_cursor_(self, cursor, is_row):
         """Add a cursor into the cursor manager to track it.
-        
+
         Args:
             cursor (Cursor): The cursor object to track.
-        
+
         Returns:
             bool: Wheather add cursor successfully.
         """
@@ -278,7 +291,7 @@ class Table():
             if not self._row_cursor_mgr.contains(cursor):
                 self._row_cursor_mgr.add_cursor(cursor)
                 self._col_cursor_mgr.on_svg_margin_changed((
-                    self._cell_margin+self._row_cursor_mgr.get_cursors_occupy(), self._cell_margin))
+                    self._cell_margin + self._row_cursor_mgr.get_cursors_occupy(), self._cell_margin))
                 need_update_svg = True
         else:
             if not self._col_cursor_mgr.contains(cursor):
@@ -293,13 +306,12 @@ class Table():
             return True
         return False
 
-
     def _remove_cursor_(self, cursor):
         """Remove one cursor from Vector and it's SVG representation.
 
         Args:
             cursor (Cursor): The cursor object to be removed.
-        
+
         Returns:
             bool: Wheather remove cursor successfully.
         """
@@ -309,12 +321,12 @@ class Table():
         if self._row_cursor_mgr.contains(cursor):
             self._row_cursor_mgr.remove_cursor(cursor)
             self._col_cursor_mgr.on_svg_margin_changed((
-                self._cell_margin+self._row_cursor_mgr.get_cursors_occupy(), self._cell_margin))
+                self._cell_margin + self._row_cursor_mgr.get_cursors_occupy(), self._cell_margin))
             need_update_svg = True
         if self._col_cursor_mgr.contains(cursor):
             self._col_cursor_mgr.remove_cursor(cursor)
             self._row_cursor_mgr.on_svg_margin_changed((
-                self._cell_margin, self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()))
+                self._cell_margin, self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()))
             need_update_svg = True
         if need_update_svg:
             self._update_svg_size_()
@@ -323,12 +335,11 @@ class Table():
             return True
         return False
 
-
     def __getitem__(self, r):
         """
         Args:
             r (int/Cursor): The index of the row to access.
-        
+
         Returns:
             TabRowIter: The iterator object for the row in the table.
 
@@ -338,11 +349,9 @@ class Table():
         """
         return TableRowOperator(r, self)
 
-    
     def __iter__(self):
         self._next_row = 0
         return self
-    
 
     def __next__(self):
         if self._next_row >= self._row:
@@ -351,7 +360,6 @@ class Table():
             res = TableRowIter(self._next_row, self)
             self._next_row += 1
             return res
-    
 
     def _repr_svg_(self):
         """
@@ -367,44 +375,45 @@ class Table():
             self._svg.update_rect_element(gid, fill=self._cell_tcs[gid].color())
             if not hold:
                 self._frame_trace_old.append((gid, color))
+        for gid, label in self._items_to_update.items():
+            self._svg.update_rect_element(gid, text=label, delay=self._delay)
+        self._items_to_update.clear()
         self._row_cursor_mgr.refresh_cursors_animation(self._row, (0, self._delay))
         self._col_cursor_mgr.refresh_cursors_animation(self._col, (0, self._delay))
         res_svg = self._svg._repr_svg_()
+        self._svg.clear_animates()
         self._frame_trace.clear()
         self._row_cursor_mgr.update_cursors_position()
         self._col_cursor_mgr.update_cursors_position()
         return res_svg
 
-
     def _update_svg_size_(self):
-        svg_width = self._col*self._cell_width + self._cell_margin*2
-        svg_height = self._row*self._cell_height + self._cell_margin*2
+        svg_width = self._col * self._cell_width + self._cell_margin * 2
+        svg_height = self._row * self._cell_height + self._cell_margin * 2
         svg_width += self._row_cursor_mgr.get_cursors_occupy()
         svg_height += self._col_cursor_mgr.get_cursors_occupy()
         if self._show_index:
-            svg_width += len(str(self._row-1))*self._label_font_size
+            svg_width += len(str(self._row - 1)) * self._label_font_size
             svg_height += self._label_font_size
         self._svg.update_svg_size(svg_width, svg_height)
 
-
     def _update_rects_position_(self):
         for r in range(self._row):
             for c in range(self._col):
-                rect_pos_x = c*self._cell_width+self._cell_margin+self._row_cursor_mgr.get_cursors_occupy()
-                rect_pos_y = r*self._cell_height+self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()
+                rect_pos_x = c * self._cell_width + self._cell_margin + self._row_cursor_mgr.get_cursors_occupy()
+                rect_pos_y = r * self._cell_height + self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()
                 rect = (rect_pos_x, rect_pos_y, self._cell_width, self._cell_height)
                 gid = self._index2rect[(r, c)]
                 self._svg.update_rect_element(gid, rect)
 
-
     def _update_subscripts_position_(self):
         for r in range(self._row):
-            pos_x = self._col*self._cell_width+self._cell_margin*2+self._row_cursor_mgr.get_cursors_occupy()
-            pos_y = (r+0.5)*self._cell_height+self._label_font_size*0.5+self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()
+            pos_x = self._col * self._cell_width + self._cell_margin * 2 + self._row_cursor_mgr.get_cursors_occupy()
+            pos_y = (r + 0.5) * self._cell_height + self._label_font_size * 0.5 + self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()
             self._svg.update_text_element(self._row_index2text[r], (pos_x, pos_y))
         for c in range(self._col):
-            pos_x = (c+0.5)*self._cell_width-self._label_font_size*len(str(c))*0.25+self._cell_margin+self._row_cursor_mgr.get_cursors_occupy()
-            pos_y = self._row*self._cell_height+1+self._label_font_size+self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()
+            pos_x = (c + 0.5) * self._cell_width - self._label_font_size * len(str(c)) * 0.25 + self._cell_margin + self._row_cursor_mgr.get_cursors_occupy()
+            pos_y = self._row * self._cell_height + 1 + self._label_font_size + self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()
             self._svg.update_text_element(self._col_index2text[c], (pos_x, pos_y))
 
     def _check_index_type_and_range_(self, index, max_val):
@@ -420,21 +429,21 @@ class Table():
         return res
 
     def _new_rect_in_svg_(self, r, c):
-        rect_pos_x = c*self._cell_width+self._cell_margin+self._row_cursor_mgr.get_cursors_occupy()
-        rect_pos_y = r*self._cell_height+self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()
+        rect_pos_x = c * self._cell_width + self._cell_margin + self._row_cursor_mgr.get_cursors_occupy()
+        rect_pos_y = r * self._cell_height + self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()
         rect = (rect_pos_x, rect_pos_y, self._cell_width, self._cell_height)
         gid = self._svg.add_rect_element(rect, self._data[r][c], angle=False)
         self._index2rect[(r, c)] = gid
         self._cell_tcs[gid] = TraceColorStack()
 
     def _new_row_index_text_in_svg_(self, r):
-        pos_x = self._col*self._cell_width+self._cell_margin*2+self._row_cursor_mgr.get_cursors_occupy()
-        pos_y = (r+0.5)*self._cell_height+self._label_font_size*0.5+self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()
+        pos_x = self._col * self._cell_width + self._cell_margin * 2 + self._row_cursor_mgr.get_cursors_occupy()
+        pos_y = (r + 0.5) * self._cell_height + self._label_font_size * 0.5 + self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()
         gid = self._svg.add_text_element((pos_x, pos_y), r, self._label_font_size)
         self._row_index2text[r] = gid
 
     def _new_col_index_text_in_svg_(self, c):
-        pos_x = (c+0.5)*self._cell_width-self._label_font_size*len(str(c))*0.25+self._cell_margin+self._row_cursor_mgr.get_cursors_occupy()
-        pos_y = self._row*self._cell_height+1+self._label_font_size+self._cell_margin+self._col_cursor_mgr.get_cursors_occupy()
+        pos_x = (c + 0.5) * self._cell_width - self._label_font_size * len(str(c)) * 0.25 + self._cell_margin + self._row_cursor_mgr.get_cursors_occupy()
+        pos_y = self._row * self._cell_height + 1 + self._label_font_size + self._cell_margin + self._col_cursor_mgr.get_cursors_occupy()
         gid = self._svg.add_text_element((pos_x, pos_y), c, self._label_font_size)
         self._col_index2text[c] = gid
